function X = graph2shape(G, parts)
% function X = graph2shape(G, [parts])
% 
% Funcio que donat un graf 'G' calculat amb la funcio 'shape2graph'
% reconstrueix la forma original 'X'. Si li passem varis grafs per files
% emmagatzema les diferents shapes en la 3era dimensio de 'X'
% 
% ENTRADES
% - G: matriu (n x p) de celes on cada cela es un node del 'n' grafs
% - parts: (opcional) vector (1 x p) de cel.les on indiquem les 
% agrupacions de landmarks a cada node. Si no ho passem suposem 
% que cada node es en un landmark
% 
% SORTIDES
% - X: vector (l x d x n) amb les 'n' shapes reconstruides segons el graf 
% 'G' i les 'parts' especificades
% 

    if nargin > 1
        if length(parts) ~= size(G,2)
            error(['El graf i les parts han de tenir el mateix nombre de components']);
        end
%         Parsejem les parts
        parts_ok = parse_parts(parts);
    end
    
    X = [];
    for i = 1:size(G,1)
        
        x = [];
        for j = 1:size(G,2)

    %         Si especifiquem 'parts'
            if nargin > 1

                part = parts_ok{j};

    %             NOTA: Si especifiquem 'parts' llavors hi ha 2 columnes !!
                x(part,:) = reshape(G{i,j},length(part),2);

    %         Si no especifiquem 'parts'
            else
                fils = size(G{i,j},1);

    %             Si no especifiquem 'parts' el node nomes pot tenir 1 o 2
    %             elements
                if fils > 2
                    error(['Les parts del graf no concorden']);
                end

                x = [x ; reshape(G{i,j},1,fils)];  % reconstruim tal qual
            end
        end
        
        X = cat(3,X,x);
    end

end
