function [X Xs] = llegir_elem_aam(dir_elem)
% function [X Xs] = llegir_elem_aam(dir_elem)
% Funcio que donat un directori llegeix tots els fitxers de punts
% continguts en els directoris comencats per 'punts' i retorna les dades de
% les configuracions (gestionant els nans)
% 
% ENTRADES:
% - dir_elem: string amb el directori que conte els directoris de punts
% 
% SORTIDES:
% - X: matriu (fils x 2 x n) amb 'n' configuracions de punts
% - Xs: matriu (fils x n) amb info pres. abs. de les 'n' configuracions
% 

%    Agafo nomes els directoris que comencen per 'punts' de dins del elem
   llista_dirpunts = dir(fullfile(dir_elem,'punts*'));
   
%    Desare els noms dels fitxers de punts. Per juntar els fitxers de punts
%    de tots els directoris en una sola matriu de punts.
   noms_punts = [];
   X = [];  % matriu de punts definitiva
   Xs = [];  % matriu info P/A
   
   for j = 1:length(llista_dirpunts)
       if ~llista_dirpunts(j).isdir
           continue;
       end
       
       dir_punts = fullfile(dir_elem,llista_dirpunts(j).name);
       
%        Agafo nomes els fitxers amb extensio .pts
       llista_punts = dir(fullfile(dir_punts,'*.pts'));
       
%        Matriu auxiliar on anire desant els resultats de les unions
%        parcials
       X_aux = [];
       Xs_aux = [];
       landmarks = [];  % nombre de landmarks del directori
       l = 1;
       
       for k = 1:length(llista_punts)
           [pathstr, name, ext, versn] = fileparts(llista_punts(k).name);
           
           fitxer_punts = fullfile(dir_punts,llista_punts(k).name);
           
%            ATENCIO: estem suposant que el primer directori conte
%            punts per a totes les imatges de la classe. Es per aquest
%            motiu que desarem els noms dels fitxers d'akest
%            directori com a referencia per tal d'ajuntar els punts de tots
%            els directoris degudament.
           if j == 1
               noms_punts = [noms_punts ; {name}];
               [X(:,:,k) landmarks] = readPointsFile(fitxer_punts);
               Xs = [Xs ones(landmarks,1)];
           else
               [X_act landmarks] = readPointsFile(fitxer_punts);
               
               while ~strcmp(name,noms_punts{l}) && l <= size(noms_punts,1)
                   X_aux = cat(3,X_aux,nan*ones(landmarks,2));
                   Xs_aux = cat(2,Xs_aux,zeros(landmarks,1));
                   l = l + 1;
               end
               if l > size(noms_punts,1)
                   error('El fitxer de punts ',fitxer_punts,...
                       'no es correspon amb cap dels anteriors');
               end
               X_aux = cat(3,X_aux,X_act);
               Xs_aux = cat(2,Xs_aux,ones(landmarks,1));
               l = l + 1;
           end
           
       end
       
%        En cas que no es tracti del primer directori de punts, acabem
%        d'omplir amb NaN's al final dels elements que faltin
%            ATENCIO: estem suposant que el primer directori conte
%            punts per a totes les imatges de la classe.
       if j~=1
           l = size(X,3) - size(X_aux,3);
           if l > 0
               X_aux = cat(3,X_aux,nan*ones(landmarks,2,l));
               Xs_aux = cat(2,Xs_aux,zeros(landmarks,l));
           end
           X = cat(1,X,X_aux);
           Xs = cat(1,Xs,Xs_aux);
       end
   end
end
