function pintar_parts(ZP,Xs,thresh,parts,mode,eixos,titol)
% pintar(ZP,Xs,thresh,modul,eixos,titol,num_mode)
% Ploteja les figures passade per ZP amb info estructural en Xs
% 
%   ENTRADES:
%   - ZP: Matriu (l x 2 x k) de 'k' figures a plotejar
%   - Xs: Matriu (l x k) amb info estructural de les figures de ZP
%   - thresh: threshold aplicat a la info estructural per decidir si pintar
%   o no pintar un punt.
%   - parts: matriu (p x 1) que defineix les 'p' parts en files de cel.les
%   - mode: 0, una sola figura desplacats; 1, una sola figura sobreposats i
%   2, varies figures. Si es un vector fila llavors posarem les figures en
%   forma de matriu.
%   - eixos: vector (1 x 4) amb les dimensions dels eixos. Opcional
%   - titol: string com a titol del plot. Opcional
% 

    if nargin < 7
        titol = [];
    end
    
    if size(mode,2) > 1
        ncols = mode(2);
        mode = 0;
    else
        ncols = 65535;
    end
    
% Si s'ha de fer en una sola figura
    if mode == 0 || mode == 1
        figure;
    end
    
    zp = [];
    max_hor = [];
    max_vert = [];
    max_v_global = [];
    n = size(ZP,3);
    col = 1;
    
    for i = 1:n 
        
%         Si s'ha de fer en una sola figura
        if mode == 0
%            Calculo el maxim horitzontal de la figura anterior i el maxim
%            vertical global (ull amb la direccio 'YDir')
            if ~isempty(zp)
                max_hor = max(max(zp(:,1,:)));
                if ~isempty(max_v_global)
                    max_v_global = max([max_v_global max(max(zp(:,2,:)))]);
                else
                    max_v_global = max(max(zp(:,2,:)));
                end
            end
        end
        
%        Construeixo les figures de cada part conforme indica la variable 'parts'
        zp = [];
        xs = [];
        for j = 1:size(parts,1)
            for k = 1:size(parts{j},2)
                zp(k,:,j) = ZP(parts{j}(k),:,i);
                xs(k,j) = Xs(parts{j}(k),i);
            end
        end

%         Si s'ha de fer en una sola figura
        if mode == 0
            
%             Comprovo si s'ha de fer translacio vertical (ncols)
            if col > ncols % INTERCANVIABLE2
%             if ~isempty(max_hor) % INTERCANVIABLE1
%                 if max_hor > ncols % INTERCANVIABLE1
                    max_hor = [];
                    max_vert = max_v_global;
%                 end % INTERCANVIABLE1
                col = 1;
            end
%             Si no es la primera figura de la fila calculo i aplico
%             translacio horitzontal
            if ~isempty(max_hor)
                min_hor = min(min(zp(:,1,:))); % minim de la forma actual
                gap = (max(max(zp(:,1,:))) - min_hor) / 10; % espai entre shapes
                translacio = max_hor - min_hor + gap; % translacio horitzontal
                zp(:,1,:) = zp(:,1,:) + translacio;
            end
%             Si no estem a la primera fila aplico translacio vertical
            if ~isempty(max_vert)
                min_vert = min(min(zp(:,2,:))); % minim de la forma actual
                gap = (max(max(zp(:,2,:))) - min_vert) / 10; % espai entre shapes
                translacio = max_vert - min_vert + gap; % translacio horitzontal
                zp(:,2,:) = zp(:,2,:) + translacio;
            end

            col = col + 1;
        end
        
%         Si s'ha de fer en varies figures
        if mode == 2
            figure;
        end
        
        for j = 1:size(parts,1)
%                 Amb o sense eixos
            if nargin < 6
                plot(zp(pos(xs(:,j),thresh),1,j),zp(pos(xs(:,j),thresh),2,j),'b.'),...
                    title(titol),hold on;
            else
                plot(zp(pos(xs(:,j),thresh),1,j),zp(pos(xs(:,j),thresh),2,j),'b.'),...
                    axis(eixos),title(titol),hold on;
            end
        end
        
        set(gca,'YDir','reverse');
    end
end