function ZP = procrustes_miss(X,siznshap,parts)
% function ZP = procrustes_miss(X,siznshap,parts)
% 
% Funcio que fa alineament procrustes a matrius amb missing landmarks
% segons ten berge et al. -> orthogonal procrustes rotation for matrices
% with missing values
% 
%   ENTRADES:
%   - X: matriu (l x 2 x n) de 'n' configuracions de 'l' landmarks en 2d.
%   - siznshap: boolea que indica si conservar la informacio del tamany (1)
%   o no (0). opcional, si no s'indica es suposara 0.
%   - parts: les parts (opcional). Per pintar informacio de debug
% 
%   SORTIDES:
%   - ZP: matriu (l x 2 x n) amb les 'n' configuracions alineades.
% 

    if nargin < 1
        error(['Has de passar 1 parametre com a minim']);
    end
    
    if size(X,2) ~= 2 || size(X,3) < 2
        error(['La matriu entrada te dimensions erronies']);
    end

    % Precisio per avaluar la convergencia
    precisio = 2;

    % Si no indiquem el tercer argument suposarem que volem eliminar la
    % informacio sobre el tamany
    if nargin < 2
        siznshap = false;
    end

    n = size(X,3);
    l = size(X,1);
    
%     Posem els punts que no hi son al punt mig
    Xs = [];  % aqui desarem les posicions on hi havia NaN's per restaurar 
             % la info al final
    for i = 1:n
        xs = ~isnan(X(:,1,i));
        
%         Control d'errors
        xs_aux = ~isnan(X(:,2,i));
        if xs ~= xs_aux 
            error(['Els NaNs han danar per parelles']);
        end
        
        p = postrue(xs);
        center_x = mean(X(p,1,i));
        center_y = mean(X(p,2,i));
        p = postrue(~xs);
        X(p,1,i) = center_x * ones(length(p),1);
        X(p,2,i) = center_y * ones(length(p),1);
        Xs = [Xs xs];
    end
    
    ZP = X;
    c = center_matrix(l);

%     Centrem 
    for i = 1:n
        ZP(:,:,i) = c*ZP(:,:,i);  % centrem 
    end

%   Si no conservem la info del tamany, calcula preformes
    if siznshap == false
    %     Preformes
        for i = 1:n
            ZP(:,:,i) = ZP(:,:,i) / centroid_size(ZP(:,:,i));
        end
    end

% % Debug        
if nargin > 2
    pintar_parts(ZP,ones(size(Xs,1),size(Xs,2)),0.5,parts,1);
end
% % 

    fi3 = false;
    
%     re-CENTRATS
    while ~fi3
        
        fi2 = false;

    %     Mentre encara es puguin fer re-ESCALATS
        while ~fi2

            fi = false;
            suma_ant = 0;

    %         Mentre encara es puguin fer ROTACIONS
            while ~fi

    %             R O T A C I O N S
                for i = 1:n
    %                 Mitja per a cada element
                    ZM = zeros([l 2]);
                    for j = 1:n
                        if j ~= i
                           ZM = ZM + ZP(:,:,j); 
                        end
                    end
                    ZM2 = ZM;
                    ZM = ZM / (n-1);

    %                 Calculem la rotacio
                    [V,S,U] = svd(ZM'*ZP(:,:,i));
                    T = U*V';

    %                 Calculem configuracio que minimitza minims quadrats per
    %                 substituir els missing landmarks
                    A = (ZM2*T') / (n-1);

    %                 Substituim els valors que falten
                    p = postrue(~Xs(:,i));
                    ZP(p,:,i) = A(p,:);

    %                 Apliquem rotacio a cada element
                    ZP(:,:,i) = ZP(:,:,i)*T;
                end

    %             Forma Mitja Procrustes de la iteracio
                u = mean(ZP,3);

    %             Calculem l'error
                suma = 0;
                for i = 1:n
                    for j = i+1:n
                        r = ZP(:,:,i) - ZP(:,:,j);
                        suma = suma + trace(r'*r);
                    end
                end

    %             Avaluem convergencia
                if int32(suma*10^precisio) == int32(suma_ant*10^precisio)
                    fi = true;
                end

                suma_ant = suma;
            end  %(fi1 rotacions)

    %         Si conservem info del tamany surt del bucle de reescalats
            if siznshap == true
                suma2_ant = suma;
                break;
            end

    %         E S C A L A T S
    %         Posem les dades en forma vectorial
            DATA = [];
            for i = 1:n
                DATA = [DATA reshape(ZP(:,:,i),2*l,1)];
            end

    %         Matriu de correlacio
            CORR = corrcoef(DATA);

    %         Calculem escalat de cada configuracio
            [V D] = eig(CORR);
            b = [];
            for i = 1:n
                suma = 0;
                for k = 1:n
                    suma = suma + trace(ZP(:,:,k)'*ZP(:,:,k));
                end
                suma = sqrt(suma / trace(ZP(:,:,i)'*ZP(:,:,i)));
                b(i) = suma*V(i,n);
            end

    %         Apliquem escalats
            for i = 1:n
                ZP(:,:,i) = b(i)*ZP(:,:,i);
            end

    %         Calcul error
            suma2 = 0;
            for i = 1:n
                for j = i+1:n
                    r = ZP(:,:,i) - ZP(:,:,j);
                    suma2 = suma2 + trace(r'*r);
                end
            end

    %         Avaluem convergencia
            if int32(suma2*10^precisio) == int32(suma_ant*10^precisio)
                fi2 = true;
            end

            suma2_ant = suma2;

        end  %fi2 (re-escalats)
        
%        Centrem 
        for i = 1:n
            ZP(:,:,i) = c*ZP(:,:,i);  % centrem 
        end

%     %   Si no conservem la info del tamany, calcula preformes
%         if siznshap == false
%         %     Preformes
%             for i = 1:n
%                 ZP(:,:,i) = ZP(:,:,i) / centroid_size(ZP(:,:,i));
%             end
%         end

% % Debug        
if nargin > 2
    pintar_parts(ZP,ones(size(Xs,1),size(Xs,2)),0.5,parts,1);
end
% % 

%         Calculem l'error
        suma3 = 0;
        for i = 1:n
            for j = i+1:n
                r = ZP(:,:,i) - ZP(:,:,j);
                suma3 = suma3 + trace(r'*r);
            end
        end

%         Avaluem convergencia
        if int32(suma3*10^precisio) == int32(suma2_ant*10^precisio)
            fi3 = true;
        end
        
    end  %re-CENTRATS (fi3)
    
%     Restaurem el NaNs
    for i = 1:n
        p = postrue(~Xs(:,i));
        ZP(p,:,i) = ones(length(p),2)*nan;
    end

end

