function [parts,n] = readPartsFile(fname)
%  readPartsFile  reads parts files generated with Tim Cootes
%                 ASM software: am_markup.
%
%  Synopsis:
%     [P,n] = readPartsFile(fname)
%   
%  Input:
%     fname  = name of the file containing the parts (required)
%
%  Output:
%     P = 1 x n struct containing each part
%     n = number of parts
%
%  Author:
%     Gerard Sanroma, gerard.sanroma@urv.cat
%     Department of Computer Engineering and Mathematics
%     Universitat Rovira i Virgili
%     17 January 2007

if nargin < 1
   error(['Need to specify a filename']);
end

fin = fopen(fname,'r');
if fin < 0
   error(['Could not open ',fname,' for input']);
end

parts = [];
n = 0;

while ~feof(fin)
    
    % Read until the begining of a Part
    buffer = fgetl(fin);
    k = strfind(buffer,'{');
    while isempty(k) && ~feof(fin)
        buffer = fgetl(fin);
        k = strfind(buffer,'{');
    end

    if length(k) > 1
        error(['Bad format file ',fname]);
    end
    if feof(fin)
        continue;
    end

    % Get the first line of the part definition
    part = [];
    part = {buffer(k+1:length(buffer))};

    % Get the rest of lines until the end of the part definition
    buffer = fgetl(fin);
    k = strfind(buffer,'}');
    while isempty(k) && ~feof(fin)
        part = [part ; {buffer}];
        buffer = fgetl(fin);
        k = strfind(buffer,'}');
    end

    if length(k) > 1
        error(['Bad format file ',fname]);
    end
    if feof(fin)
        continue;
    end

    % Get the last line of the part definition
    part = [part ; {buffer(1:k-1)}];

    % Read until finds 'indices' tag
    i = 1;
    k = strfind(part{i},'indices');
    while isempty(k) && (i < length(part))
        i = i+1;
        k = strfind(part{i},'indices');
    end

    if isempty(k)
        error(['Bad format file ',fname]);
    end

    % Get the delimiters of the substring containing the data
    ini = strfind(part{i},'(');
    fi = strfind(part{i},')');

    if isempty(ini) || isempty(fi) || (length(ini) > 1) || (length(fi) > 1)
        error(['Bad format file ',fname]);
    end

    % Parse the string into a vector
    indices = strread(strrep(part{i}(ini+1:fi-1),',',' '),'%u')';
    indices = indices+1;  % because matlab starts from 1 (not from 0)

    % Read until finds 'form' tag
    i = 1;
    k = strfind(part{i},'form');
    while isempty(k) && (i < length(part))
        i = i+1;
        k = strfind(part{i},'form');
    end

    if isempty(k)
        error(['Bad format file ',fname]);
    end

    % Parse string
    boundary = strread(part{i},'%s');
    
    if boundary{2}(1:6) == 'closed'
        indices = [indices indices(1)];
    end

%     parts = [parts struct('indices',indices,'boundary',boundary(2))];
    parts = [parts {indices}];
    n = n+1;

end