function [P,n] = readPointsFile(fname)
%  readPointsFile  reads points data files generated with Tim Cootes
%                  ASM points annotation software: am_markup.
%
%  Synopsis:
%     [P,n] = readPointsFile(fname)
%   
%  Input:
%     fname  = name of the file containing the data (required)
%
%  Output:
%     P = n x 2 matrix of points.
%     n = number of points
%
%  Author:
%     Gerard Sanroma, gerard.sanroma@urv.cat
%     Department of Computer Engineering and Mathematics
%     Universitat Rovira i Virgili
%     17 January 2007

if nargin < 1
   error(['Need to specify a filename']);
end

fin = fopen(fname,'r');
if fin < 0
   error(['Could not open ',fname,' for input']);
end

% Read until finds 'n_points' tag
buffer = fgetl(fin);
k = strfind(buffer,'n_points');
while isempty(k)
    buffer = fgetl(fin);
    k = strfind(buffer,'n_points');
end

if length(k) > 1
    error(['Bad format file ',fname]);
end

% Get number of points
[aux, n] = strread(buffer,'%s%u');  % get the number of points

% Read until the begining of data
buffer = fgetl(fin);
k = strfind(buffer,'{');
while isempty(k)
    buffer = fgetl(fin);
    k = strfind(buffer,'{');
end

if length(k) > 1
    error(['Bad format file ',fname]);
end

P = fscanf(fin,'%f',2*n);  % read data
P = reshape(P,2,n)';  % reshape data

fclose(fin);
