function G = shape2graph(X, parts)
% function G = shape2graph(X, [parts])
% 
% Funcio que donada una forma 'X' en calcula el graf 'G' segons la
% informacio en 'parts'. 'X' pot tenir valors nuls (definits amb 'NaN').
% Si li passem mes d'una shape en la tercera dimensio desa un graf en cada
% fila
% 
% ENTRADES
% - X: matriu (l x d x n) de 'n' shapes amb 'l' valors (landmarks) en 'd' 
% dimensions 
% - parts: (opcional) vector (1 x p) de cel.les on indiquem les 
% agrupacions de landmarks que farem. Si no ho passem suposem que prenem 
% cada landmark per separat
% 
% SORTIDES
% - G: matriu (n x p) de 'n' grafs de 'p' cel.les si especifiquem 'parts'
% o (n x l) en cas contrari. Cada cela conte un vector columna amb els 
% atributs de cada node.
% 

    if nargin < 1
        error(['Has de passar almenys un argument']);
    end
    
    cols = size(X,2);
    
    if cols > 2
        error(['El nombre de columnes no pot ser superior a 2']);
    end

%     Si especifiquem 'parts'
    if nargin > 1
        if cols ~= 2
            error(['Si especifiquem parts, les columnes han de ser 2']);
        end
%         Parsejem les parts per eliminar possibles repetits
        parts_ok = parse_parts(parts);
    else
%         Sino, ens construim les nostres, amb un landmark a cada part
        parts_ok = [];
        for i = 1:size(X,1)
            parts_ok = [parts_ok {[i]}];
        end
    end
    
    G = [];
    for i = 1:size(X,3)
        
        g = [];
        for j = 1:length(parts_ok)
            part = parts_ok{j};

    %        En cada part han de ser tots o cap NaN
            if ~isnan(X(part(1),1,i))
                if ~comp(isnan(X(part,:,i)),...
                        logical(false*ones(length(part),cols)))
                    error(['Les parts han de tenir o tots o cap nan']);
                end
            else
                if ~comp(isnan(X(part,:,i)),...
                        logical(true*ones(length(part),cols)))
                    error(['Les parts han de tenir o tots o cap nan']);
                end
            end

    %         Reestructurem dades en columna [x;..;x ; y;..;y]
            g = [g {reshape(X(part,:,i),length(part)*cols,1)}];
        end
        
        G = [G ; g];
    end

end
